package com.qmth.teachcloud.exchange.common.contant;

import com.qmth.boot.core.uid.service.UidService;
import com.qmth.teachcloud.exchange.common.util.Base64Util;
import org.apache.commons.io.IOUtils;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.nio.charset.Charset;
import java.util.Objects;
import java.util.UUID;

/**
 * @Description: 系统常量
 * @Param:
 * @return:
 * @Author: wangliang
 * @Date: 2019/10/11
 */
public class SystemConstant {

    /**
     * 系统常量
     */
    public static final String SYS_CONFIG_KEY_CHARSETS = "sys.txt.charset";
    public static final String CHARSET_NAME = "UTF-8";
    //    public static final String CHARSET_GB2312 = "gb2312";
    public static final String CHARSET_GBK = "gbk";
    public static final Charset CHARSET = Charset.forName(CHARSET_NAME);
    public static final String SUCCESS = "success";
    public static final String UPDATE_TIME = "updateTime";
    public static final String SESSION = "session:";
    public static final String USER = "account";
    public static final String USER_COMMON = "accountCommon";
    public static final String SCHOOL_ID = "schoolId";
    public static final String SCHOOL = "school";
    public static final String SCHOOL_CODE = "schoolCode";
    public static final String SCHOOL_CODE_COMMON = "schoolCodeCommon";
    public static final String RETURN_URL = "returnUrl";
    public static final String RETURN_URL_COMMON = "returnUrlCommon";
    public static final String PARAMS = "params";
    public static final String PARAMS_COMMON = "paramsCommon";
    public static final String ORG = "org";
    public static final String MD5 = "MD5";
    public static final String ID = "id";
    public static final String SIGNATURE = "signature";
    public static final String GET = "get";
    public static final String GET_UNKNOWN = "?";
    public static final String GET_SYMBOL = "&";
    public static final String GET_EQUAL = "=";
    public static final String PATH_MATCH = "*";
    public static final String PATH_SUBSTR = "/#";
    public static final String JUMP_API_URL = "/api/open/common/authentication/teachcloud";

    /**
     * 缓存配置
     */
    public static final String SCHOOL_CACHE = "school:cache";
    public static final String SCHOOL_CODE_CACHE = "school:code:cache";
    public static final String AUTH_INFO_CACHE = "auth:info:cache";

    /**
     * 鉴权
     */
    public static final String HEADER_AUTHORIZATION = "Authorization";
    public static final String HEADER_TIME = "time";
    public static final String HEADER_PLATFORM = "platform";
    public static final String HEADER_DEVICE_ID = "deviceId";
    public static final String TOKEN = "token";
    public static final String SIGN = "sign";

    /**
     * redis
     */
    public static final int DEFAULT_SESSION_EXPIRE = 1;//过期时间1天
//    public static final long REDIS_DEFAULT_EXPIRE_TIME = 24 * 60L * 60L;//过期时间24小时

    /**
     * aes相关
     */
    public static final String AES = "AES";
    public static final String AES_MODE_PKCS5 = "AES/CBC/PKCS5Padding";//用这个模式，规则必须为16位
    public static final String AES_MODE_PKCS7 = "AES/CBC/PKCS7Padding";//用这个模式，规则必须为16位
    public static final String AES_RULE = "1234567890123456";//aes密钥

    /**
     * 线程池配置
     */
    public static final String THREAD_POOL_NAME = "taskThreadPool";
    //    public static final int THREAD_POOL_CORE_POOL_SIZE = 5;
//    public static final int THREAD_POOL_MAX_POOL_SIZE = 100;
    public static final int THREAD_POOL_KEEP_ALIVE_SECONDS = 10;
    public static final int THREAD_POOL_QUEUE_CAPACITY = 500;

    public static final String LOG_ERROR = "请求出错:{}";

    /**
     * http设置
     */
    public static final int CONNECT_TIME_OUT = 1000 * 60 * 2;//请求超时
    public static final int SOCKET_CONNECT_TIME_OUT = 1000 * 60 * 30;//读取数据超时

    /**
     * 获取数据库uuid
     *
     * @return
     */
    public static Long getDbUuid() {
        UidService uidservice = SpringContextHolder.getBean(UidService.class);
        return uidservice.getId();
    }

    /**
     * 获取全局uuid
     *
     * @return
     */
    public static String getUuid() {
        return String.valueOf(UUID.randomUUID()).replaceAll("-", "");
    }

    /**
     * 图片转换
     *
     * @param imgBase64
     * @param filePath
     * @throws IOException
     */
    public static void base64ToImage(String imgBase64, String filePath) throws IOException {
        OutputStream out = null;
        try {
            byte[] b = Base64Util.decode(imgBase64);
            out = new FileOutputStream(filePath);
            IOUtils.write(b, out);
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            if (Objects.nonNull(out)) {
                out.flush();
                out.close();
            }
        }
    }
}